package com.jpattern.service.cache.simple;

import java.io.Serializable;
import java.util.LinkedHashMap;
import java.util.Map;

import com.jpattern.service.cache.ACache;
import com.jpattern.service.cache.statistics.ICacheStatistics;

/**
 * A simple fixed size cache.
 * @author Francesco Cina
 *
 * 23 Sep 2011
 */
public class FixedSizeCache extends ACache {

	private static final long serialVersionUID = 1L;
	private Map<String, Serializable> map;
	public final int maxSize; 
	
	public FixedSizeCache(String name, int maxSize, ICacheStatistics cacheStatistics) {
		super(name, cacheStatistics);
		this.maxSize = maxSize;
		
		map = new LinkedHashMap<String, Serializable>() {
			private static final long serialVersionUID = 1L;
			
            protected boolean removeEldestEntry(Map.Entry<String, Serializable> oldest) {
                return size() > FixedSizeCache.this.maxSize;
            }
			
		};
	}
	
	@Override
	public synchronized Serializable getValue(String key) {
		return map.get(key);
	}

	@Override
	public synchronized void put(String key, Serializable value) {
		map.put(key, value);
	}

	@Override
	public synchronized void remove(String key) {
		map.remove(key);
	}

	@Override
	public synchronized void clear() {
		map.clear();
	}

	@Override
	public boolean contains(String key) {
		return map.containsKey(key);
	}

	@Override
	public int getSize() {
		return map.size();
	}
	
}
