package com.jpattern.service.cache.statistics;

import java.util.HashMap;
import java.util.Map;

/**
 * 
 * @author Francesco Cina'
 *
 * 5 May 2011
 * 
 * This class collect a Map with all the statistics of the cache.
 * DO NOT use in production! It store all the values in memory and never check
 * for the map size.
 */
public class InMemoryCacheStatistics implements ICacheStatistics {

	private static final long serialVersionUID = 1L;
	private Map<String, ICacheStatisticsEntry> statistics = new HashMap<String, ICacheStatisticsEntry>();

	@Override
	public void addEntry(String key, Object value) {
		if (!getStatistics().containsKey(key)) {
			getStatistics().put(key, new CacheStatisticsEntry());
		}
		update( getStatistics().get(key), value);
	}

	private void update(ICacheStatisticsEntry iCacheStatisticsEntry, Object value) {
		if (value==null) {
			iCacheStatisticsEntry.increaseMissing();
		} else {
			iCacheStatisticsEntry.increaseHits();
		}
	}

	@Override
	public ICacheStatisticsEntry getStatistics(String key) {
		if (statistics.containsKey(key)) {
			return statistics.get(key);
		}
		return new NullCacheStatisticsEntry();
	}

	@Override
	public Map<String, ICacheStatisticsEntry> getStatistics() {
		return statistics;
	}

}
